#!/usr/bin/perl

#    cclip is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.

#    cclip is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.

#    You should have received a copy of the GNU General Public License
#    along with cclip.  If not, see <http://www.gnu.org/licenses/>.

#-----------------------------------------------------------------------------------------#

# Author: Federico Salotti
# Organization/Company: Mr_Robot_Computers_Repair 
# Country: Italy Barga(LU)

# release: 0.1 (30 May 2020 15:27)
# initial release

# current: 0.2 (05 January 2022 14:41)
# add Wayland support

use warnings;
use feature "say";
use Getopt::Long;
use MIME::Base64 qw(encode_base64);
use Encode;

GetOptions
(
	"str=s" => \$str,
	"encode=s{,}" => \@encodes,
	"eol" => \$eol,
	"verbose" => \$verbose,
	"msg=s" => \$msg,
	"help" => \$help
);

$str=undef unless $str;
$msg="string copied into clipboard" unless $msg;

if($eol){$str.="\n";}

sub check_xclip
{
	my @execs=glob("/usr/bin/*");
	foreach (@execs)
	{
		if(/xclip/) {return 1;}
	}
	return 0;
}

sub check_wl_clipboard
{
	my @execs=glob("/usr/bin/*");
	foreach (@execs)
	{
		if(/wl-*/) {return 1;}
	}
	return 0;
}

sub check_x_session
{
	if( "x11" eq $ENV{XDG_SESSION_TYPE}){return 1;}
	return 0;
}

sub rot1Encode
{
	$_[0] =~ tr/B-ZA-Ab-za-a/A-Za-z/;
	return $_[0];
}

sub rot2Encode
{
	$_[0] =~ tr/C-ZA-Bc-za-b/A-Za-z/;
	return $_[0];
}

sub rot3Encode
{
	$_[0] =~ tr/D-ZA-Cd-za-c/A-Za-z/;
	return $_[0];
}

sub rot4Encode
{
	$_[0] =~ tr/E-ZA-De-za-d/A-Za-z/;
	return $_[0];
}

sub rot5Encode
{
	$_[0] =~ tr/F-ZA-Ef-za-e/A-Za-z/;
	return $_[0];
}

sub rot6Encode
{
	$_[0] =~ tr/G-ZA-Fg-za-f/A-Za-z/;
	return $_[0];
}

sub rot7Encode
{
	$_[0] =~ tr/H-ZA-Gh-za-g/A-Za-z/;
	return $_[0];
}

sub rot8Encode
{
	$_[0] =~ tr/I-ZA-Hi-za-h/A-Za-z/;
	return $_[0];
}

sub rot9Encode
{
	$_[0] =~ tr/J-ZA-Ij-za-i/A-Za-z/;
	return $_[0];
}

sub rot10Encode
{
	$_[0] =~ tr/K-ZA-Jk-za-j/A-Za-z/;
	return $_[0];
}

sub rot11Encode
{
	$_[0] =~ tr/L-ZA-Kl-za-k/A-Za-z/;
	return $_[0];
}

sub rot12Encode
{
	$_[0] =~ tr/M-ZA-Lm-za-l/A-Za-z/;
	return $_[0];
}

sub rot13Encode
{
	$_[0] =~ tr/N-ZA-Mn-za-m/A-Za-z/;
	return $_[0];
}

sub rot14Encode
{
	$_[0] =~ tr/O-ZA-No-za-n/A-Za-z/;
	return $_[0];
}

sub rot15Encode
{
	$_[0] =~ tr/P-ZA-Op-za-o/A-Za-z/;
	return $_[0];
}

sub rot16Encode
{
	$_[0] =~ tr/Q-ZA-Pq-za-p/A-Za-z/;
	return $_[0];
}

sub rot17Encode
{
	$_[0] =~ tr/R-ZA-Qr-za-q/A-Za-z/;
	return $_[0];
}

sub rot18Encode
{
	$_[0] =~ tr/S-ZA-Rs-za-r/A-Za-z/;
	return $_[0];
}

sub rot19Encode
{
	$_[0] =~ tr/T-ZA-St-za-s/A-Za-z/;
	return $_[0];
}

sub rot20Encode
{
	$_[0] =~ tr/U-ZA-Tu-za-t/A-Za-z/;
	return $_[0];
}

sub rot21Encode
{
	$_[0] =~ tr/V-ZA-Uv-za-u/A-Za-z/;
	return $_[0];
}

sub rot22Encode
{
	$_[0] =~ tr/W-ZA-Vw-za-v/A-Za-z/;
	return $_[0];
}

sub rot23Encode
{
	$_[0] =~ tr/X-ZA-Wx-za-w/A-Za-z/;
	return $_[0];
}

sub rot24Encode
{
	$_[0] =~ tr/Y-ZA-Xy-za-x/A-Za-z/;
	return $_[0];
}

sub rot25Encode
{
	$_[0] =~ tr/Z-ZA-Yz-za-y/A-Za-z/;
	return $_[0];
}

sub help
{
	say "=== [ cclip v.0.2 ] ===";
	say "--str < string to copy in the clipboard >";
	say "--encode < string encode type rot[1-25] , base64 >";
	say "--eol < include end of line in encode >";
	say "--msg < set output message default=string copied into clipboard >";
	say "--verbose < verbose output >";
}

if(check_x_session)
{
	if(!check_xclip) {say "[!] ERROR: xclip is not installed in your system. Install xclip!";exit;}
}
else
{
	if(!check_wl_clipboard) {say "[!] ERROR: wl-clipboard is not installed in your system. Install wl-clipboard!";exit;}
}

if(!$str){ say "[!] Enter a string!";help;exit;}
elsif($help){help;exit;}
else
{
	$encode_str=$str;
	
	if(scalar @encodes != 0)
	{
		foreach $encode (@encodes)
		{
			if(lc $encode eq lc 'rot1') {$encode_str=rot1Encode($encode_str);}
			if(lc $encode eq lc 'rot2') {$encode_str=rot2Encode($encode_str);}
			if(lc $encode eq lc 'rot3') {$encode_str=rot3Encode($encode_str);}
			if(lc $encode eq lc 'rot4') {$encode_str=rot4Encode($encode_str);}
			if(lc $encode eq lc 'rot5') {$encode_str=rot5Encode($encode_str);}
			if(lc $encode eq lc 'rot6') {$encode_str=rot6Encode($encode_str);}
			if(lc $encode eq lc 'rot7') {$encode_str=rot7Encode($encode_str);}
			if(lc $encode eq lc 'rot8') {$encode_str=rot8Encode($encode_str);}
			if(lc $encode eq lc 'rot9') {$encode_str=rot9Encode($encode_str);}
			if(lc $encode eq lc 'rot10') {$encode_str=rot10Encode($encode_str);}
			if(lc $encode eq lc 'rot11') {$encode_str=rot11Encode($encode_str);}
			if(lc $encode eq lc 'rot12') {$encode_str=rot12Encode($encode_str);}
			if(lc $encode eq lc 'rot13') {$encode_str=rot13Encode($encode_str);}
			if(lc $encode eq lc 'rot14') {$encode_str=rot14Encode($encode_str);}
			if(lc $encode eq lc 'rot15') {$encode_str=rot15Encode($encode_str);}
			if(lc $encode eq lc 'rot16') {$encode_str=rot16Encode($encode_str);}
			if(lc $encode eq lc 'rot17') {$encode_str=rot17Encode($encode_str);}
			if(lc $encode eq lc 'rot18') {$encode_str=rot18Encode($encode_str);}
			if(lc $encode eq lc 'rot19') {$encode_str=rot19Encode($encode_str);}
			if(lc $encode eq lc 'rot20') {$encode_str=rot20Encode($encode_str);}
			if(lc $encode eq lc 'rot21') {$encode_str=rot21Encode($encode_str);}
			if(lc $encode eq lc 'rot22') {$encode_str=rot22Encode($encode_str);}
			if(lc $encode eq lc 'rot23') {$encode_str=rot23Encode($encode_str);}
			if(lc $encode eq lc 'rot24') {$encode_str=rot24Encode($encode_str);}
			if(lc $encode eq lc 'rot25') {$encode_str=rot25Encode($encode_str);}
			if(lc $encode eq lc 'base64') {$encode_str=encode_base64($encode_str);}
		}
	}
	# X11 session
	if(check_x_session) { system("echo '$encode_str' | xclip -sel clip"); }

	# Wayland session
	else {system("echo '$encode_str' | wl-copy"); }

	if($verbose)
	{
		chomp $str;

		say "pclip v0.2";
		$verbose_output="\n[+] command --str $str";

		if(scalar @encodes != 0) {$verbose_output.=" --encode @encodes";}
		if($eol){$verbose_output.=" --eol";}
		if($verbose){$verbose_output.=" --verbose";}
		$verbose_output.=" --msg $msg";

		say $verbose_output;
	}
	else {say $msg;}
}
